/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Adapter.Widgets.JoinWidget : Gtk.Grid {

    Gtk.Button connect_button;
    Gtk.ComboBoxText server_entry;
    Gtk.ComboBoxText id_entry;
    Gtk.Entry channel_entry;

    public signal void done (string address, string nick_name, string channel);

    public JoinWidget () {
        setup_widgets ();
        fill_in_information ();
        connect_signals ();

        channel_entry.grab_focus ();
        channel_entry.set_position (-1);
    }

    void setup_widgets () {
        margin = 12;
        column_spacing = 6;
        row_spacing = 6;

        var server_label = new Gtk.Label (_("Network:"));
        server_label.halign = Gtk.Align.END;
        server_label.valign = Gtk.Align.CENTER;

        var id_label = new Gtk.Label (_("Identity:"));
        id_label.halign = Gtk.Align.END;
        id_label.valign = Gtk.Align.CENTER;

        var channel_label = new Gtk.Label (_("Channel:"));
        channel_label.halign = Gtk.Align.END;
        channel_label.valign = Gtk.Align.CENTER;

        server_entry = new Gtk.ComboBoxText.with_entry ();

        id_entry = new Gtk.ComboBoxText.with_entry ();

        channel_entry = new Gtk.Entry ();
        channel_entry.text = "#";
        channel_entry.width_request = 250;

        connect_button = new Gtk.Button.with_label (_("Connect"));
        connect_button.get_style_context ().add_class ("suggested-action");
        connect_button.sensitive = false;

        var connect_button_grid = new Gtk.Grid ();

        var connect_fake_label = new Gtk.Label ("");
        connect_fake_label.hexpand = true;

        connect_button_grid.attach (connect_fake_label, 0, 0, 1, 1);
        connect_button_grid.attach (connect_button, 1, 0, 1, 1);

        attach (server_label, 0, 0, 1, 1);
        attach (server_entry, 1, 0, 1, 1);
        attach (id_label, 0, 1, 1, 1);
        attach (id_entry, 1, 1, 1, 1);
        attach (channel_label, 0, 2, 1, 1);
        attach (channel_entry, 1, 2, 1, 1);
        attach (connect_button_grid, 1, 3, 1, 1);
    }

    void fill_in_information () {
        for (int i = 0; i < Utils.Network.all.size; i++)
            server_entry.append (i.to_string (), Utils.Network.all[i].address);
        server_entry.active = 0;

        for (int i = 0; i < Utils.Identity.all.size; i++) {
            id_entry.append (i.to_string (), Utils.Identity.all[i].nick_name);
            if (Utils.Identity.all[i].nick_name == Settings.preferences.last_used_nick)
                id_entry.active = i;
        }

        if (Settings.preferences.last_used_nick == "")
            id_entry.active = 0;
    }

    void connect_signals () {
        id_entry.changed.connect (set_connect_button_sensitivity);
        server_entry.changed.connect (set_connect_button_sensitivity);
        channel_entry.changed.connect (set_connect_button_sensitivity);
        connect_button.clicked.connect (on_activated);
        channel_entry.activate.connect (on_activated);
    }

    void set_connect_button_sensitivity () {
        connect_button.sensitive = input_is_valid ();
    }

    bool input_is_valid () {
        return channel_entry.text.length > 1
            && server_entry.get_active_text ().length > 0
            && id_entry.get_active_text ().length > 0;
    }

    void on_activated () {
        var network = Utils.Network.get_or_create (server_entry.get_active_text ());
        var identity = Utils.Identity.get_or_create (id_entry.get_active_text ());

        var channel_with_hash = channel_entry.text;
        if (channel_with_hash [0:1] != "#")
             channel_with_hash = "#" + channel_entry.text;

        done (network.address, identity.nick_name, channel_with_hash);
    }
    
    public void focus_channel_entry () {
        channel_entry.grab_focus ();
        channel_entry.set_position (-1);
    }
}
