/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Controller.Server : GLib.Object {

    public weak Window window;
    public Port.View.Server view;
    public Port.Irc.Server backend;

    public string nick; // FIXME not reliable
    public string address; // FIXME not reliable
    public string name; // id

    public Gee.HashMap <string, Channel> channels {
        get; set; default = new Gee.HashMap <string, Channel> ();
    }

    public Gee.HashMap <string, PrivateChat> private_chats {
        get; set; default = new Gee.HashMap <string, PrivateChat> ();
    }

    public Server (string address, string nick, Window window) {
        this.backend = window.backend.get_or_create_server (address, nick);
        this.name = backend.name;
        this.window = window;
        this.nick = nick;
        this.address = address;

        initialize ();
        connect_signals ();
    }

    void initialize () {
        if (!backend.connected)
            backend.send_connect ();

        var network_name = Utils.Network.get_or_create (address).name;
        var view_name = @"$network_name - $nick";
        this.view = window.view.add_server (view_name, backend.name);
    }

    void connect_signals () {
        view.toggle_away.connect (toggle_backend_away_status);
        view.part_all.connect (part_all_channels);
        backend.event_user_away.connect (set_frontend_away);
        backend.event_quit.connect (frontend_show_quit);
        backend.event_message.connect (frontend_show_message);
        backend.event_nick.connect (change_frontend_nick);
    }

    void change_frontend_nick (string from, string new_nick) {
        if (backend.is_me (from)) {
            var network_name = Utils.Network.get_or_create (address).name;
            var view_name = @"$network_name - $new_nick";
            view.name = view_name;
        }
    }

    void toggle_backend_away_status () {
        if (backend.away) {
            backend.send_back ();
        } else {
            backend.send_away ();
        }
    }

    void part_all_channels () {
        foreach (var channel in channels.values)
            if (nick in channel.backend.nicks)
                channel.backend.send_part ();
    }

    void set_frontend_away (string from, bool away) {
        if (backend.is_me (from)) {
            view.away = away;
        }

        var nick = from.split ("!")[0];

        foreach (var channel in channels.values)
            if (nick in channel.backend.nicks)
                channel.view.set_away_status (nick, away);
    }

    void frontend_show_quit (string from, string message) {
        var nick = from.split ("!")[0];

        foreach (var channel in channels.values)
            if (nick in channel.backend.nicks)
                channel.view.remove_user (nick, message);
    }

    void frontend_show_message (string from, string target, string message) {
        if (target != nick)
            return;

        var from_nick = from.split ("!")[0];
        open_new_chat_with (from_nick, message);
    }

    public void open_new_chat_with (string from_nick, string message = "") {
        if (from_nick == backend.nick)
            return;

        if (!private_chats.has_key (from_nick)) {
            var chat = new PrivateChat (this, from_nick, message);
            private_chats [from_nick] = chat;
        }
    }
}
